`timescale 1ns / 1ps

//******************************************************************************
//* MiniRISC CPU v2.0                                                          *
//*                                                                            *
//* A mveleteket vgrehajt adatstruktra.                                    *
//******************************************************************************
module datapath(
   //rajel.
   input  wire       clk,
   
   //Az adatmemrival kapcsolatos jelek.
   output wire [7:0] data_mem_addr,    //Cmbusz
   input  wire [7:0] data_mem_din,     //Olvassi adatbusz
   output wire [7:0] data_mem_dout,    //rsi adatbusz
   
   //Az utastsban lv konstans adat.
   input  wire [7:0] const_data,
   
   //A multiplexerek vezrl jelei.
   input  wire       wr_data_sel,      //A regiszterbe rand adat kivlasztsa
   input  wire       addr_op2_sel,     //Az ALU 2. operandusnak kivlasztsa
   
   //A regisztertmbbel kapcsolatos jelek.
   input  wire [3:0] reg_addr_x,       //Regiszter cme (X port)
   input  wire [3:0] reg_addr_y,       //Regiszter cme (Y port)
   input  wire       reg_wr_en,        //rs engedlyez jel
   
   //Az ALU-val kapcsolatos jelek.
   input  wire [1:0] alu_op_type,      //ALU mvelet kivlaszt jel
   input  wire [1:0] alu_arith_sel,    //Aritmetikai mvelet kivlaszt jel
   input  wire [1:0] alu_logic_sel,    //Logikai mvelet kivlaszt jel
   input  wire [3:0] alu_shift_sel,    //Shiftelsi mvelet kivlaszt jel
   input  wire [3:0] alu_flag_din,     //A flag-ekbe rand rtk
   input  wire       alu_flag_wr,      //A flag-ek rs engedlyez jele
   output wire       alu_flag_z,       //Zero flag
   output wire       alu_flag_c,       //Carry flag
   output wire       alu_flag_n,       //Negative flag
   output wire       alu_flag_v,       //Overflow flag
   
   //A programszmll j rtke ugrs esetn.
   output wire [7:0] jump_address,
   
   //A debug interfsz jelei.
   input  wire [7:0] dbg_addr_in,      //Cm bemenet
   input  wire [7:0] dbg_data_in,      //Adatbemenet
   input  wire       dbg_is_brk,       //A trspont llapot jelzse
   output wire [7:0] dbg_reg_dout      //A regisztertmbbl beolvasott adat
);

//******************************************************************************
//* A regisztermbbe rand adatot kivlaszt multiplexer.                     *
//******************************************************************************
wire [7:0] alu_result;                 //Az ALU mvelet eredmnye.
reg  [7:0] reg_wr_data;                //A regisztertmbbe rand adat.

always @(*)
begin
   case ({dbg_is_brk, wr_data_sel})
      2'b00  : reg_wr_data <= alu_result;
      2'b01  : reg_wr_data <= data_mem_din;
      default: reg_wr_data <= dbg_data_in;
   endcase
end


//******************************************************************************
//* A regisztertmb.                                                           *
//******************************************************************************
wire [7:0] reg_rd_data_x;
wire [7:0] reg_rd_data_y;
wire [3:0] address_x = (dbg_is_brk) ? dbg_addr_in[3:0] : reg_addr_x;

reg_file reg_file(
   //rajel.
   .clk(clk),
   
   //Az rsi s az X olvassi port.
   .addr_x(address_x),                 //A regiszter cme
   .write_en(reg_wr_en),               //rs engedlyez jel
   .wr_data_x(reg_wr_data),            //A regiszterbe rand adat
   .rd_data_x(reg_rd_data_x),          //A regiszterben trolt adat
   
   //Az Y olvassi port.
   .addr_y(reg_addr_y),                //A regiszter cme
   .rd_data_y(reg_rd_data_y)           //A regiszterben trolt adat
);

//A memria rsi adatbusznak meghajtsa.
assign data_mem_dout = (dbg_is_brk) ? dbg_data_in : reg_rd_data_x;

//A regisztertmbbl beolvasott adat a debug interfsz fel.
assign dbg_reg_dout  = reg_rd_data_x;


//******************************************************************************
//* Az ALU 2. operandust s a memriacmet kivlaszt multiplexer.            *
//* -addr_op2_sel=0: konstans / abszolt cmzs                                *
//* -addr_op2_sel=1: regiszter / indirekt cmzs                               *
//******************************************************************************
wire [7:0] alu_operand2 = (addr_op2_sel) ? reg_rd_data_y : const_data;

//A memria cmbusznak meghajtsa.
assign data_mem_addr    = (dbg_is_brk) ? dbg_addr_in : alu_operand2;

//A programszmll j rtke ugrs esetn.
assign jump_address     = alu_operand2;


//******************************************************************************
//* Az aritmetikai-logikai egysg (ALU).                                       *
//******************************************************************************
alu alu(
   //rajel.
   .clk(clk),
   
   //Vezrl bemenetek.
   .op_type(alu_op_type),              //Az ALU mvelet tpust kivlaszt jel
   .arith_sel(alu_arith_sel),          //Az aritmetikai mveletet kivlaszt jel
   .logic_sel(alu_logic_sel),          //A logikai mveletet kivlaszt jel
   .shift_sel(alu_shift_sel),          //A shiftelsi mveletet kivlaszt jel
   
   //Az operandusok s az eredmny.
   .operand1(reg_rd_data_x),           //Els operandus
   .operand2(alu_operand2),            //Msodik operandus
   .result(alu_result),                //Az ALU mvelet eredmnye
   
   //ALU felttel jelek.
   .flag_din(alu_flag_din),            //A flag-ekbe rand rtk
   .flag_wr(alu_flag_wr),              //A flag-ek rs engedlyez jele
   .flag_z(alu_flag_z),                //Zero flag
   .flag_c(alu_flag_c),                //Carry flag
   .flag_n(alu_flag_n),                //Negative flag
   .flag_v(alu_flag_v)                 //Overflow flag
);

endmodule
