`timescale 1ns / 1ps

//******************************************************************************
//* Visszaolvashat 8 bites kimeneti modul.                                    *
//*                                                                            *
//* A perifria cme paramter tadssal llthat be a fels szint modulban. *
//* a GPIO modul megpldnyostsakor. A szintzis mr ennek megfeleleen az   *
//* aktulis BASEADDR bziscmmel trtnik.                                    *
//*                                                                            *
//* A processzor rsi (WRITE) ciklusnak iddiagramja:                        *
//* Az rsi ciklust az 1 rajel ciklus ideig aktv CPU2DMEM_WR jel jelzi. Az  *
//* rsi ciklus ideje alatt a cm s a kimeneti adat stabil.                  *
//*                                                                            *
//*                     --------          --------          --------           *
//* CLK                |        |        |        |        |        |          *
//*                ----          --------          --------          ----      *
//*                                        ------------------                  *
//* CPU2DMEM_WR                           /                  \                 *
//*               ------------------------                    -----------      *
//*               ------------------------ ------------------ -----------      *
//* CPU2DMEM_ADDR                         X   RVNYES CM   X                 *
//*               ------------------------ ------------------ -----------      *
//*               ------------------------ ------------------ -----------      *
//* CPU2DMEM_DATA                         X   RVNYES ADAT  X                 *
//*               ------------------------ ------------------ -----------      *
//*                                                        ^                   *
//*                                Az rsi parancs vgrehajtsa itt trtnik. *
//*                                                                            *
//* A processzor olvassi (READ) ciklusnak iddiagramja:                      *
//* Az olvassi ciklust az 1 rajel ciklus ideig aktv CPU2DMEM_RD jel jelzi.  *
//* Az olvassi ciklus ideje alatt a cm stabil s a kivlasztott perifria az *
//* olvassi adatbuszra kapuzza az adatot. Az olvassi ciklus eltt s utn    *
//* az olvassi adatbusz rtke inaktv 0 kell, hogy legyen.                   *
//*                                                                            *
//*                     --------          --------          --------           *
//* CLK                |        |        |        |        |        |          *
//*                ----          --------          --------          ----      *
//*                                        ------------------                  *
//* CPU2DMEM_RD                           /                  \                 *
//*               ------------------------                    -----------      *
//*               ------------------------ ------------------ -----------      *
//* CPU2DMEM_ADDR                         X   RVNYES CM   X                 *
//*               ------------------------ ------------------ -----------      *
//*               ------------------------ ------------------ -----------      *
//* DMEM2CPU_DATA            0            X   RVNYES ADAT  X       0         *
//*               ------------------------ ------------------ -----------      *
//*                                                        ^                   *
//*                              A bemeneti adat mintavtelezse itt trtnik. *  
//******************************************************************************
module basic_owr #(
   //A perifria bziscme.
   parameter BASEADDR = 8'hff
) (
   //rajel s reset.
   input  wire       clk,              //rajel
   input  wire       rst,              //Reset jel
   
   //A busz interfsz jelei.
   input  wire [7:0] cpu2dmem_addr,    //Cmbusz
   input  wire       cpu2dmem_wr,      //rs engedlyez jel
   input  wire       cpu2dmem_rd,      //Olvass engedlyez jel
   input  wire [7:0] cpu2dmem_data,    //rsi adatbusz
   output wire [7:0] dmem2cpu_data,    //Olvassi adatbusz
   
   //A GPIO interfsz jelei.
   output wire [7:0] gpio_out          //Az IO lbakra kirand adat
);
  
//******************************************************************************
//* Cmdekdols.                                                              *
//******************************************************************************
//A perifria kivlaszt jele.
wire psel = (cpu2dmem_addr == BASEADDR);

//A kimeneti adatregiszter rsnak s olvassnak jelzse.
wire out_reg_wr = psel & cpu2dmem_wr;
wire out_reg_rd = psel & cpu2dmem_rd;


//******************************************************************************
//* A kimeneti adatregiszter.                                                  *
//*                                                                            *
//* Az alapfunkcit egyszer adatkiadssal valstjuk meg. A kimeneti adat     *
//* egy regiszterbe kerl, ez fogja meghajtani a kimeneti vonalakat. Teht a   *
//* kimeneti buszon egy rsi ciklus ideig megjelen  dinamikus a regiszter    *
//* egy statikus, stabil adatt alaktja (amg jra nem rjuk).                *
//******************************************************************************
reg [7:0] out_reg;

always @(posedge clk)
begin
   if (rst)
      out_reg <= 8'd0;                 //Reset esetn trljk a regisztert
   else
      if (out_reg_wr)
         out_reg <= cpu2dmem_data;     //Kimeneti adatregiszter rsa
end

//A kimeneti lbak meghajtsa.
assign gpio_out = out_reg;


//******************************************************************************
//* A processzor olvassi adatbusznak meghajtsa. Az olvassi adatbuszra csak *
//* az olvass ideje alatt kapcsoljuk r a krt rtket, egybknt egy inaktv *
//* nulla rtk jelenik meg rajta (elosztott busz multiplexer funkci).        *
//******************************************************************************
assign dmem2cpu_data = (out_reg_rd) ? out_reg : 8'd0;

endmodule
